/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Straße
 * D-76571 Gaggenau
 * T. +49 (0)7225 / 64-0
 * F. +49 (0)7225 / 64-300
 */

package de.bsvrz.vew.sim.sim;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.sim.sim.verwaltung.SimulationsVarianten;
import de.bsvrz.vew.sim.sim.verwaltung.VerwaltungSimulation;

/**
 * Applikationsklasse der SWE Simulation.<br>
 * Für den übergebenen Konfigurationsbereich (und optional Konfigurationsobjekt
 * vom Typ typ.simulationsVerwaltung) wird eine Instanz von <code>VerwaltungSimulation</code>
 * erzeugt und initialisiert.
 *
 * @author Dambach Werke GmbH
 * @author Stefan Sans
 * @version $Revision: 1.3 $ / $Date: 2008/10/31 13:30:31 $ / ($Author: Sans $)
 */
public class SimulationApp implements StandardApplication

{
  /**
   * Debug-Logger für Logging-Ausgaben
   */
  private Debug _debug;

   /**
   * -konfigBereich von Kommandozeile
   */
  private String _konfigBereich = null;

  /**
   * -konfigObjekt von Kommandozeile
   */
  private String _konfigObjekt = null;

  /**
   * Simulationsverwaltungs-Funktionalität
   */
  private VerwaltungSimulation _simuVerw;

  /**
   * -anmeldeTimeout von Kommandozeile
   */
  private int _anmeldeTimeout = 250;

  /**
   * Datenverteiler-Verbindung
   */
  private ClientDavInterface _connection;


  /**
   * Wird das Archiv in der Version vor Archiv-Version 3 benutzt?
   */
  private static boolean _archivVorV3 = false;

  /**
   * @return wird das Archiv in der Version vor Archiv-Version 3 benutzt?
   */
  public static boolean isArchivVorV3()
  {
    return _archivVorV3;
  }


  /**
   * Default- Konstruktor.<br>
   * Wird typischerweise von <code>main</code> aufgerufen
   */
  public SimulationApp()
  {
    try
    {
      Thread.sleep(100);
    }
    catch (InterruptedException e)
    {
    }
  }
  /**
   * Extrahiert - nach Prüfung - den Konfigurationsbereich aus dem Attribut {@link #_konfigBereich}
   * @return den gefunden Konfigurationsbereich
   * @throws IllegalArgumentException, falls Konfigurationsbereich nicht gefunden
   */
  private ConfigurationArea getKonfigurationsBereich()
  {
    ConfigurationArea ca = null;
    if (_konfigBereich != null)
    {
      ca = _connection.getDataModel().getConfigurationArea(_konfigBereich);
      if (ca == null)
      {
        throw new IllegalArgumentException("Zur PID \"" + _konfigBereich +
          "\" wurde kein Konfigurationsbereich gefunden ");
      }
    }
    else
    {
      ConfigurationObject co = _connection.getDataModel().getConfigurationAuthority();
      AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften");
      Data data = co.getConfigurationData(atg);
      if (data != null)
      {
        TextArray array = data.getTextArray("defaultBereich");
        if (array.getLength() > 0)
        {
          ca = (ConfigurationArea)_connection.getDataModel().getObject(array.getText(0));
        }
      }
      if (ca == null)
      {
        throw new IllegalArgumentException("Für das Konfigurationsobjekt " + co.getPid() +
            " konnte kein Default-Konfigurationsbereich bestimmt werden");
      }
    }
    return ca;
  }

  /**
   * Extrahiert - nach Prüfung - das Konfigurationsobjekt für die Simulationsverwaltung
   * aus dem Attribut {@link #_konfigObjekt}
   * @return das Konfigurationsobjekt zur Simulationsverwaltung
   * @throws IllegalArgumentException, falls Konfigurationsobjekt nicht gefunden
   * oder vom falschen Typ
   */
  private ConfigurationObject getKonfigurationsObjekt()
  {
    ConfigurationObject co = null;
    if (_konfigObjekt != null)
    {
      try
      {
        co = (ConfigurationObject)_connection.getDataModel().getObject(_konfigObjekt);
      }
      catch(ClassCastException e)
      {
        throw new IllegalArgumentException("Die PID \"" + _konfigObjekt +
        "\" bezeichnet kein KonfigurationsObjekt");
      }
      if (co == null)
      {
        throw new IllegalArgumentException("Zur PID \"" + _konfigObjekt +
        "\" wurde kein Konfigurationsobjekt gefunden ");
      }
    }
    else
    {
      co = _connection.getLocalConfigurationAuthority();
    }
    if (!co.isOfType("typ.simulationsVerwaltung"))
    {
      throw new IllegalArgumentException("Das als Konfigurationsobjekt für die " +
          "Simulationsverwaltung vorgesehene Objekt \"" + co.getPid() +
          "\" ist nicht vom Typ typ.simulationsVerwaltung");
    }
    return co;
  }

  /**
   * Überschriebene Methode von <code>StandardApplication</code>,
   * die die speziellen Startparameter auswertet.<br>
   * Die Argumwente, die ausgewertet werden, sind durch die Attribute {@link #_konfigBereich},
   * {@link #_konfigObjekt} und {@link #_anmeldeTimeout} - alle optional - spezifiziert
   *
   * @param argumentList
   *  siehe sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
   */
  public void parseArguments(ArgumentList argumentList)
  {
    _debug = Debug.getLogger();

    if (argumentList.hasArgument("-konfigBereich"))
    {
      _konfigBereich = argumentList.fetchArgument("-konfigBereich=").asNonEmptyString();
    }

    if (argumentList.hasArgument("-konfigObjekt"))
    {
      _konfigObjekt = argumentList.fetchArgument("-konfigObjekt=").asNonEmptyString();
    }

     if (argumentList.hasArgument("-anmeldeTimeout"))
     {
       _anmeldeTimeout = argumentList.fetchArgument("-anmeldeTimeout=").intValueBetween(1, 60000);
     }

     if (argumentList.hasArgument("-archivVorV3"))
     {
       _archivVorV3 = true;
     }

    argumentList.fetchUnusedArguments();
  }


  /**
   * Überschriebene Methode von StandardApplication, die die Initialisierung durchführt.<br>
   * Es wird eine Instanz von <code>VerwaltungSimulation</code> erzeugt und initialisiert
   *
   * @param connection
   * siehe sys.funclib.application.StandardApplication#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
   */
  public void initialize(ClientDavInterface connection)
  {
    _connection = connection;
    try
    {
      IntegerAttributeType att =
        (IntegerAttributeType)connection.getDataModel().getAttributeType("att.simulationsVariante");
      short maximum = (short)att.getRange().getMaximum();
      SimulationsVarianten.getInstanz().setMaximum(maximum);
      _debug.config("Maximale Simulationsvariante ist " + maximum);

      _simuVerw = new VerwaltungSimulation(connection,
          getKonfigurationsBereich(), getKonfigurationsObjekt());

      _simuVerw.anmelden();
      if (_simuVerw.warteAnmeldungVewAnfrGueltig(_anmeldeTimeout))
      {
        _simuVerw.initialisiereVorhandeneSimulationen();
        _debug.info("SWE Simulation initialisiert");
      }
      else
      {
        _debug.error("Start der SWE Simulation abgebrochen wegen ungültiger Anmeldung");
        System.exit(1);
      }
    }
    catch (Exception e)
    {
    	//e.printStackTrace();
    	_debug.error("Abbruch der SWE Simulation während der Initialisierung", e);
      System.exit(2);
    }
  }


  /**
   * Programmeinstieg.<br>
   *
   * @param arguments Kommandozeilenargumente, folgende speziellen Argumente werden ausgewertet:<p>
   * -konfigBereich: Konfigurationsbereich, in dem die dynamischen Simulationsobjkete verwaltet werden<p>
   * -konfigObjekt: Konfigurationsobjekt für die Verwaltung von Simulationen - muss vom Typ
   * typ.simulationsVerwaltung sein. Optional, wenn nicht angegeben, wird das Konfigurationsobjekt des
   * lokalen Konfigurationsverantwortlichen genommen.<p>
   * -anmeldeTimeout: Maximale Wartezeit für (neg.) Quittung auf Anmeldung als Senke für Objekt vom
   * Typ typ.SimulationsVerwaltung.Optional, nur zu benutzten, wenn Default nicht ausreichend gross
  */
  public static void main(String[] arguments)
  {
    StandardApplicationRunner.run(new SimulationApp(), arguments);
  }
  
//  /**
//   * Sendet eine Betriebsmeldung vom Meldungstyp APPLICATION_DOMAIN mit der 
//   * Meldungsklasse WARNING
//   *  
//   * @param message Meldungstext
//   * @param reference referenziertes Systemobjekt
//   */
//  protected void sendMessage(String message, SystemObject reference)
//  {
//    String id = "" + getConnection().getLocalApplicationObject().getId();
//    sendMessage(id, MessageType.APPLICATION_DOMAIN, MessageGrade.WARNING, message, reference);
//  }
//  
//  /**
//   * Sendet eine Betriebsmeldung
//   *  
//   * @param type Meldungstyp
//   * @param grade Meldungsklasse
//   * @param message Meldungstext
//   * @param reference referenziertes Systemobjekt
//   */ 
//  protected void sendMessage(String id, MessageType type, MessageGrade grade, String message, 
//      SystemObject reference)
//  {
//    MessageSender.getInstance().sendMessage(id, type, null, grade, reference,  
//        MessageState.MESSAGE, new MessageCauser(getConnection().getLocalUser(), "", ""), message);
//  }
  
}
