/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.sim.sim.steuerung;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.sim.sim.sender.Sender;
import de.bsvrz.vew.sim.sim.simulation.Simulation;

/**
 * Verwaltung der parametrierenden ATG "SimulationStartInfo" 
 * für eine Simulation
 *
 * @version  $Revision: 1.1 $ / $Date: 2008/10/31 13:30:30 $ / ($Author: Sans $)
 * @author  Dambach-Werke GmbH
 * @author  Stefan Sans
 */
public class StartInfoVerwaltung
{
  /**
   * Debug-Logger für Logging-Ausgaben
   */
  private static final Debug _debug = Debug.getLogger();

  /**
   * Neue Instanz einer StartInfo-Verwaltung
   *
   * @param simulation die Simulatin, für die die
   * neue Instanz erzeugt werden soll
   */
  public StartInfoVerwaltung(Simulation simulation)
  {
    _simulation = simulation;
  }

  /**
   * Die Simulation, deren StartInfo-Parameter hier verwaltet werden
   */
  protected final Simulation _simulation;

  /**
   * Bestimmen der aktuellen StartInfo-Parameter.
   * Alle aktuellen StartInfos in einer Map: Schlüssel ist jeweils die PID
   * des Rechnerobjkets, Wert ist die Startskript-Spezifikation
   * @return eine Map mit allen StartInfo-Daten
   */
  public Map<String, String> getStartInfo()
  {
    final Map<String, String> startInfo = new LinkedHashMap<String, String>();
    ClientReceiverInterface empf = new ClientReceiverInterface()
    {
      public void update(ResultData[] resultData)
      {
        for (ResultData data : resultData)
        {
          _debug.finest("Daten empfangen:\n" + data);
          if (data.getDataState() == DataState.DATA)
          {
            AttributeGroup atg = data.getDataDescription().getAttributeGroup();
            Aspect asp = data.getDataDescription().getAspect();
            short sv = data.getDataDescription().getSimulationVariant();

            _debug.finer("Daten empfangen mit atg/asp/sv: " + atg.getPid() + "/" +
                asp.getPid() + "/" + sv + ":\n" + data);

            if (/* sv == getVarianteFuerSimulation() && */
                asp.getPid().equals("asp.parameterSoll") &&
                atg.getPid().equals("atg.simulationStartInfo"))
            {
              _debug.finer("Bearbeite Daten mit atg/asp/sv: " + atg.getPid() + "/" +
                  asp.getPid() + "/" + sv + ":\n" + data);

              Data datensatz = data.getData();
              Array array = data.getData().getArray("StartInfoListe");
              for (int i = 0; i < array.getLength(); i += 1)
              {
                datensatz = array.getItem(i);
                ReferenceValue refValue = datensatz.getReferenceValue("Rechner");
                if (refValue.getSystemObject() != null)
                {
                  TextValue textValue = datensatz.getTextValue("BlockId");
                  startInfo.put(refValue.getSystemObject().getPid(), textValue.getText());
                }
              }
            }
          }
          synchronized(this)
          {
            notify();
          }
        }
      }
    };

    ClientDavInterface con = _simulation.getConnection();
    ReceiveOptions anmeldeoptionen = ReceiveOptions.normal();
    ReceiverRole empfaengerrolle = ReceiverRole.receiver();
    AttributeGroup attributgruppe = con.getDataModel().getAttributeGroup("atg.simulationStartInfo");
    Aspect aspekt = con.getDataModel().getAspect("asp.parameterSoll");
    final long EMPFANGS_TIMEOUT = 2000;

    DataDescription datenbeschreibung = new DataDescription(attributgruppe, aspekt /*, simulationsvariante */);

    con.subscribeReceiver(empf, _simulation.getObjekt(),
       datenbeschreibung, anmeldeoptionen, empfaengerrolle);

    _debug.fine("Warte max. " + EMPFANGS_TIMEOUT + "ms auf parametrierende Daten für Simulation " + this);
    synchronized(empf)
    {
      try
      {
        empf.wait(EMPFANGS_TIMEOUT);
      }
      catch(InterruptedException e)
      {
        // tue nichts
      }
    }
    con.unsubscribeReceiver(empf, _simulation.getObjekt(), datenbeschreibung);
    return startInfo;
  }

  /**
   * Die StartInfo-Parameter aktualisieren.
   * Die StartInfo-Parameter der zugeordneten Simulation werden durch die
   * übergebenen Daten ersetzt
   *
   * @param startInfo eine Map: Schlüssel ist jeweils die PID
   * des Rechnerobjkets, Wert ist die StartStopp-Start-Id
   */
  public void aktualisiereStartInfo(Map<String, String> startInfo)
  {
    ClientDavInterface con = _simulation.getConnection();
    AttributeGroup atg = con.getDataModel().getAttributeGroup("atg.simulationStartInfo");
    Aspect asp = con.getDataModel().getAspect("asp.parameterVorgabe");

    Data data = con.createData(atg);
    Array array = data.getArray("StartInfoListe");
    Iterator it = startInfo.keySet().iterator();

    while(it.hasNext())
    {
      String pid = (String)it.next();
      String id = (String)startInfo.get(pid);
      SystemObject rechner = con.getDataModel().getObject(pid);
      if (rechner != null && rechner.isOfType("typ.rechner"))
      {
        array.setLength(array.getLength() + 1);
        Data datensatz = array.getItem(array.getLength() - 1);
        ReferenceValue refValue = datensatz.getReferenceValue("Rechner");
        refValue.setSystemObject(rechner);
        TextValue textValue = datensatz.getTextValue("BlockId");
        textValue.setText(id);
      }
      else
      {
        _debug.error("Parameter-Aktualisierung Simulation" + this + ": " +
            pid + " bezeichnet kein gültiges Rechner-Objekt");
      }
    }

    _debug.fine("Sende Parameter-Aktualisierung für Simulation " + this);
    if (Sender.send(con, _simulation.getObjekt(), atg, asp, /* getVarianteFuerSimulation() , */ data))
    {
      _debug.fine("Parameter-Aktualisierung für Simulation " + this +
          " erfolgreich gesendet");
    }
    else
    {
      _debug.error("Fehler beim Senden Parameter-Aktualisierung für Simulation " + this);
    }
  }
}
